{
  "graphitePort": 2003,
  "graphiteHost": "graphite.host.com",
  "port": 8125,
  "mgmt_port": 8126,
  "backends": [ "Console" ],
  "repeater": [ { "host": "10.8.3.214", "port": 8125 } ],
  "repeaterProtocol": "udp4"
}
